/*
 * $Id: Communication.cs 288 2007-06-06 09:06:51Z lloydd $
 */

using System;
using System.Text;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Mono.Unix;

namespace WiMo.RAPI
{
   public class CommunicationBase
   {
      protected Connection connection;
      
      protected uint last_error;
      protected uint last_result;

      protected byte[] write_buffer;
      protected byte[] read_buffer;
      protected int write_index;
      protected int read_index;
      
      public CommunicationBase (Connection connection)
      {
         this.connection = connection;

         LastError = 0xffffffff;
         LastResult = 0xffffffff;
            
         write_index = 0;
         write_buffer = new byte [0];

         read_index = 0;
         read_buffer = new byte [0];
      }

      protected int ReadIndex {
         get {
            return read_index;
         }
         set {
            if (value < 0)
               read_index = 0;
            else {
               if (value > read_buffer.Length)
                  read_index = read_buffer.Length;
               else
                  read_index = value;
            }
         }
      }

      protected int WriteIndex {
         get {
            return write_index;
         }
         set {
            if (value < 0)
               write_index = 0;
            else
               write_index = value;
         }
      }
      
      public uint LastError {
         get {
            return last_error;
         }
         set {
            last_error = value;
         }
      }
      
      public uint LastResult {
         get {
            return last_result;
         }
         set {
            last_result = value;
         }
      }
      
      public string StatusMessage {
         get {
            return string.Format ("Error: 0x{0} Result: 0x{1}",
               LastError.ToString ("X8"), LastResult.ToString ("X8"));
         }
      }
      
      public string OperationFailedMessage {
         get {
            return "Operation failed. " + StatusMessage;
         }
      }
      
      public Connection Connection
      {
         get {
            return connection;
         }
      }
      
      public NetworkStream Stream {
         get {
            return connection.Stream;
         }
      }

      public bool Debug {
         get {
            return connection.Debug;
         }
      }
      
      public void PrintIndexes ()
      {
         Console.WriteLine ("ReadIndex: {0} WriteIndex: {1}",
            ReadIndex, WriteIndex);
      }
      
      public void PrintWriteBuffer ()
      {
         PrintBuffer ("WriteBuffer", write_buffer);
      }
      
      public void PrintReadBuffer ()
      {
         PrintBuffer ("ReadBuffer", read_buffer);
      }

      protected void PrintBuffer (string text, byte[] buf)
      {
         Console.WriteLine ("{0}({1}): ", text, buf.Length);
         
         for (int i = 0; i < buf.Length; i++)
            Console.Write ("{0} {1}", buf [i].ToString ("X2"), ((i+1) % 16 == 0 ? "\n" : ""));
         
         Console.WriteLine ();
      }
   }
}
