/*
 * $Id: Context.cs 288 2007-06-06 09:06:51Z lloydd $
 */

using System;
using System.Text;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Mono.Unix;

namespace WiMo.RAPI
{
   public class Context2 : Communication2, ISystem
   {
      private int CEDB_MAXDBASENAMELEN = 32;
      private int CEDB_MAX_SORT_ORDER = 4;
      private int CEPROPVAL_SIZE = sizeof (uint) + 2 * sizeof (short) + 8;
      
      protected int rapi_version;
      
      public Context2 (Connection connection) : base (connection)
      {
         rapi_version = 2;
      }

      public int RapiVersion {
         get {
            return rapi_version;
         }
      }

      public int MaxDbaseNameLength {
         get {
            return CEDB_MAXDBASENAMELEN;
         }
      }
      
      public int MaxSortOrder {
         get {
            return CEDB_MAX_SORT_ORDER;
         }
      }
      
      public bool CeGetStoreInformation (out StoreInformation si)
      {
         si = new StoreInformation ();
         
         ClearWrite ();
         
         WriteUInt32 (0x39);
         
         Process ();
         ReadLastError ();
         ReadResult ();
         
         if (LastResult != 1)
            return false;

         si.StoreSize = ReadUInt32 ();
         si.FreeSize = ReadUInt32 ();
         
         return true;
      }
      
      public SystemInfo CeGetSystemInfo ()
      {
         uint requested_size = 7 * sizeof (uint) + 4 * sizeof (ushort);
         
         ClearWrite ();
         
         WriteUInt32    (0x3d);
         WriteOptional  (requested_size, false);
         
         Process ();
         ReadLastError ();
         
         SystemInfo si = new SystemInfo ();
         si.ProcessorArch = (ProcessorArch) ReadUInt16 ();
         si.Reserved = ReadUInt16 ();
         si.PageSize = ReadUInt32 ();
         si.MinimumApplicationAddress = ReadUInt32 ();
         si.MaximumApplicationAddress = ReadUInt32 ();
         si.ActiveProcessorMask = ReadUInt32 ();
         si.NumberOfProcessors = ReadUInt32 ();
         si.ProcessorType = (ProcessorType) ReadUInt32 ();
         si.AllocationGranularity = ReadUInt32 ();
         si.ProcessorLevel = ReadUInt16 ();
         si.ProcessorRevision = ReadUInt16 ();
         
         return si;
      }
      
      public bool CeGetSystemPowerStatusEx (out SystemPowerStatusEx ps, bool refresh)
      {
         ps = new SystemPowerStatusEx ();
         
         ClearWrite ();
         
         WriteUInt32    (0x49);
         WriteInt32     (refresh ? 1 : 0);
         
         Process ();
         ReadResult ();
         
         if (LastResult != 1)
            return false;
         
         ps.AcLineStatus = (ACStatus) ReadByte ();
         ps.BatteryFlag = (BatteryFlag) ReadByte ();
         ps.BatteryLifePercent = ReadByte ();
         ps.Reserved1 = ReadByte ();
         ps.BatteryLifeTime = ReadUInt32 ();
         ps.BatteryFullLifeTime = ReadUInt32 ();
         ps.Reserved2 = ReadByte ();
         ps.BackupBatteryFlag = (BatteryFlag) ReadByte ();
         ps.BackupBatteryLifePercent = ReadByte ();
         ps.Reserved3 = ReadByte ();
         ps.BackupBatteryLifeTime = ReadUInt32 ();
         ps.BackupBatteryFullLifeTime = ReadUInt32 ();
         
         return true;
      }
      
      public bool CeGetVersionEx (out VersionInfo vi)
      {
         ClearWrite ();
         
         vi = new VersionInfo ();
         
         WriteUInt32 (0x43);
         
         Process ();
         ReadLastError ();
         ReadResult ();

         if (LastResult != 1)
            return false;
         
         ReadUInt32 (); // size
         ReadUInt32 (); // OsVersionInfoSize
         
         vi.MajorVersion = ReadUInt32 ();
         vi.MinorVersion = ReadUInt32 ();
         vi.BuildNumber = ReadUInt32 ();
         vi.Platform = (PlatformType) ReadUInt32 ();
         
         return true;
      }
      
      public bool CeSyncTimeToPc ()
      {
         ClearWrite ();
         
         WriteUInt32    (0x01);
         WriteFiletime  (Filetime.Now);
         WriteUInt32    (0);
         WriteUInt32    (10000);
         
         Process ();
         ReadLastError ();
         ReadResult ();
         
         return LastResult == 1;
      }
      
      public bool CeCreateProcess (string application_name,
                                   string command_line,
                                   out ProcessInformation process_info)
      {
         process_info = new ProcessInformation ();

         ClearWrite ();
         
         WriteUInt32 (0x2a);
         WriteString (application_name);
         WriteString (command_line);
         WriteUInt32 (0);
         WriteUInt32 (0);
         WriteUInt32 (0);
         WriteUInt32 (0); // creation flags
         WriteUInt32 (0);
         WriteUInt32 (0);
         WriteUInt32 (0);
         
         Process ();
         ReadLastError ();
         ReadResult ();
         
         if (LastResult != 1)
            return false;
         
         process_info.Process = ReadUInt32 ();
         process_info.Thread = ReadUInt32 ();
         process_info.ProcessId = ReadUInt32 ();
         process_info.ThreadId = ReadUInt32 ();
         
         return true;
      }

      public bool CeGetSystemMemoryDivision (out uint storage_pages,
                                             out uint ram_pages,
                                             out uint page_size)
      {
         storage_pages = 0;
         ram_pages = 0;
         page_size = 0;
         
         throw new Exception ("RAPI2 CeGetSystemMemoryDivision not implemeneted");
      }

      public bool CeSetSystemMemoryDivision (uint storage_pages)
      {
         throw new Exception ("RAPI2 CeSetSystemMemoryDivision not implemeneted");
      }
      
      public bool CeCheckPassword (string password)
      {
         throw new Exception ("RAPI2 CeCheckPassword not implemented");
      }

      public bool CeKillAllApps ()
      {
         throw new Exception ("RAPI2 CeKillAllApps not implemented");
      }

      public bool CeRegCopyFile (string filename)
      {
         throw new Exception ("RAPI2 CeRegCopyFile not implemented");
      }
      
      public bool CeRegRestoreFile (string filename)
      {
         throw new Exception ("RAPI2 CeRegRestoreFile not implemented");
      }
   }
}
