/*
 * $Id: Types.cs 289 2007-06-12 09:27:37Z lloydd $
 */

using System;

namespace WiMo.RAPI
{
   public interface IValueId
   {
      uint Value {get;}
   }

   public interface IValueHandle
   {
      uint Value {get;}
   }
   
   public struct PropId : IValueId, IEquatable<PropId>
   {
      private static readonly PropId zero = new PropId (0);
      private uint value;
      private ushort id;
      private PropValType val_type;

      public PropId (ushort id, PropValType val_type)
      {
         this.value = ((uint) id << 16) + (uint) val_type;
         this.id = id;
         this.val_type = val_type;
      }

      public PropId (uint value)
      {
         this.value = value;
         this.id = (ushort) (value >> 16);
         this.val_type = (PropValType) ((ushort)(value & 0xffff));
      }

      public uint Value {
         get {
            return value;
         }
      }
      
      public PropValType ValType {
         get {
            return val_type;
         }
      }
      
      public ushort Id {
         get {
            return id;
         }
      }
      
      public static PropId Zero {
         get {
            return zero;
         }
      }
      
      public override int GetHashCode ()
      {
         return (int) Value;
      }

      public override bool Equals (object o)
      {
         if (!(o is PropId))
            return false;
         
         return Value == ((PropId)o).Value;
      }

      public bool Equals (PropId o)
      {
         return Value == o.Value;
      }

      public static bool operator ==(PropId a, PropId b) {
         return a.Equals (b);
      }

      public static bool operator !=(PropId a, PropId b) {
         return !(a.Equals (b));
      }
   }
   
   public struct FileHandle : IValueHandle, IEquatable<FileHandle>
   {
      private static readonly FileHandle zero = new FileHandle (0);
      private uint value;

      public FileHandle (uint value)
      {
         this.value = value;
      }

      public uint Value {
         get {
            return value;
         }
      }
      
      public static FileHandle Zero {
         get {
            return zero;
         }
      }
      
      public override int GetHashCode ()
      {
         return (int) Value;
      }

      public override bool Equals (object o)
      {
         if (!(o is FileHandle))
            return false;
         
         return Value == ((FileHandle) o).Value;
      }

      public bool Equals (FileHandle o)
      {
         return Value == o.Value;
      }

      public static bool operator ==(FileHandle a, FileHandle b) {
         return a.Equals (b);
      }

      public static bool operator !=(FileHandle a, FileHandle b) {
         return !(a.Equals (b));
      }
   }
   
   public struct FileFindHandle : IValueHandle, IEquatable<FileFindHandle>
   {
      private static readonly FileFindHandle zero = new FileFindHandle (0);
      private uint value;

      public FileFindHandle (uint value)
      {
         this.value = value;
      }

      public uint Value {
         get {
            return value;
         }
      }
      
      public static FileFindHandle Zero {
         get {
            return zero;
         }
      }
      
      public override int GetHashCode ()
      {
         return (int) Value;
      }

      public override bool Equals (object o)
      {
         if (!(o is FileFindHandle))
            return false;
         
         return Value == ((FileFindHandle) o).Value;
      }

      public bool Equals (FileFindHandle o)
      {
         return Value == o.Value;
      }

      public static bool operator ==(FileFindHandle a, FileFindHandle b) {
         return a.Equals (b);
      }

      public static bool operator !=(FileFindHandle a, FileFindHandle b) {
         return !(a.Equals (b));
      }
   }
   
   public struct DbHandle : IValueHandle, IEquatable<DbHandle>
   {
      private static readonly DbHandle zero = new DbHandle (0);
      private uint value;

      public DbHandle (uint value)
      {
         this.value = value;
      }

      public uint Value {
         get {
            return value;
         }
      }
      
      public static DbHandle Zero {
         get {
            return zero;
         }
      }
      
      public override int GetHashCode ()
      {
         return (int) Value;
      }

      public override bool Equals (object o)
      {
         if (!(o is DbHandle))
            return false;
         
         return Value == ((DbHandle) o).Value;
      }

      public bool Equals (DbHandle o)
      {
         return Value == o.Value;
      }

      public static bool operator ==(DbHandle a, DbHandle b) {
         return a.Equals (b);
      }

      public static bool operator !=(DbHandle a, DbHandle b) {
         return !(a.Equals (b));
      }
   }
   
   public struct DbFindHandle : IValueHandle, IEquatable<DbFindHandle>
   {
      private static readonly DbFindHandle zero = new DbFindHandle (0);
      private uint value;

      public DbFindHandle (uint value)
      {
         this.value = value;
      }

      public uint Value {
         get {
            return value;
         }
      }
      
      public static DbFindHandle Zero {
         get {
            return zero;
         }
      }
      
      public override int GetHashCode ()
      {
         return (int) Value;
      }

      public override bool Equals (object o)
      {
         if (!(o is DbFindHandle))
            return false;
         
         return Value == ((DbFindHandle) o).Value;
      }

      public bool Equals (DbFindHandle o)
      {
         return Value == o.Value;
      }

      public static bool operator ==(DbFindHandle a, DbFindHandle b) {
         return a.Equals (b);
      }

      public static bool operator !=(DbFindHandle a, DbFindHandle b) {
         return !(a.Equals (b));
      }
   }
   
   public struct Oid : IValueId, IEquatable<Oid>
   {
      private static readonly Oid zero = new Oid (0);
      private uint value;

      public Oid (uint value)
      {
         this.value = value;
      }

      public uint Value {
         get {
            return value;
         }
      }
      
      public static Oid Zero {
         get {
            return zero;
         }
      }
      
      public override int GetHashCode ()
      {
         return (int) Value;
      }

      public override bool Equals (object o)
      {
         if (!(o is Oid))
            return false;
         
         return Value == ((Oid) o).Value;
      }

      public bool Equals (Oid o)
      {
         return Value == o.Value;
      }

      public static bool operator ==(Oid a, Oid b) {
         return a.Equals (b);
      }

      public static bool operator !=(Oid a, Oid b) {
         return !(a.Equals (b));
      }
   }
   
   public class Filetime
   {
      uint HighDateTime;
      uint LowDateTime;
      
      public Filetime (DateTime dt)
      {
         Int64 t = dt.ToFileTime ();
         Int64 tt = t;
         tt >>= 32;
         HighDateTime = (uint) tt;
         tt <<= 32;
         LowDateTime = (uint)(t - tt);
      }
      
      public Filetime (uint HighDateTime, uint LowDateTime)
      {
         this.HighDateTime = HighDateTime;
         this.LowDateTime = LowDateTime;
      }
      
      public static Filetime Now {
         get {
            return new Filetime (DateTime.Now);
         }
      }
      
      public uint High {
         get {
            return HighDateTime;
         }
      }

      public uint Low {
         get {
            return LowDateTime;
         }
      }
      
      public DateTime DateTime {
         get {
            Int64 t = HighDateTime;
            t <<= 32;
            t += LowDateTime;
            return DateTime.FromFileTime (t);
         }
      }
   }
}
