/*
 * $Id: TestDatabase.cs 289 2007-06-12 09:27:37Z lloydd $
 */

#define test3

using System;
using System.Threading;
using WiMo.RAPI;

public class TestDatabaseClass
{
   public static void PrintName (string s)
   {
      Console.WriteLine ("---------------------------");
      Console.WriteLine ("--- {0}", s);
      Console.WriteLine ("---------------------------");
   }
   
   public static void Main (string[] args)
   {
      Connection connection = null;
      Context ctx = null;
      
      try {
         Console.WriteLine ("Connecting...");
         
         connection = new Connection ("10.100.1.2");
         ctx = connection.CreateContext ();
      }
      catch (Exception e) {
         Console.Error.WriteLine ("Connect failed: {0}", e.Message);
         return;
      }
      
      Console.WriteLine ("Connected");

      DbFindHandle find_handle = DbFindHandle.Zero;
      Oid oid = Oid.Zero;
      string test_db_name = "test";

      FindDatabaseAttribute flags =
         FindDatabaseAttribute.Oid|
         FindDatabaseAttribute.Flags|
         FindDatabaseAttribute.Name|
         FindDatabaseAttribute.Type|
         FindDatabaseAttribute.Num_Records|
         FindDatabaseAttribute.Num_Sort_Order|
         FindDatabaseAttribute.Size|
         FindDatabaseAttribute.Last_Modified|
         FindDatabaseAttribute.Sort_Spec;
      
#if test0
      PrintName ("CeFindAllDatabases");

      DbaseFindData[] find_data = ctx.Database.CeFindAllDatabases (0, flags);
      
      Console.WriteLine ("Found {0} databases", find_data.Length);
      
      foreach (DbaseFindData d in find_data) {
         Console.WriteLine ("----------");
         Console.WriteLine ("  Oid: \"{0}\"", d.Oid.Value);
         Console.WriteLine ("  Flags: \"{0}\"", d.DbInfo.Flags);
         Console.WriteLine ("  DbaseName: \"{0}\"", d.DbInfo.DbaseName);
         Console.WriteLine ("  DbaseType: \"{0}\"", d.DbInfo.DbaseType);
         Console.WriteLine ("  NumRecords: \"{0}\"", d.DbInfo.NumRecords);
         Console.WriteLine ("  NumSortOrder: \"{0}\"", d.DbInfo.NumSortOrder);
         Console.WriteLine ("  Size: \"{0}\"", d.DbInfo.Size);
         Console.WriteLine ("  LastModified: \"{0}\"", d.DbInfo.LastModified.DateTime);
         Console.WriteLine ("  SortSpecs.Count: \"{0}\"", d.DbInfo.SortSpecs.Length);
         
         foreach (SortOrderSpec s in d.DbInfo.SortSpecs) {
            Console.WriteLine ("  -------");
            Console.WriteLine ("    PropId: \"{0}\"", s.PropId.Value);
            Console.WriteLine ("    Flags: \"{0}\"", s.Flags);
         }

         test_db_name = "test";
         
         if (d.DbInfo.DbaseName == test_db_name) {
            PrintName ("CeSetDatabaseInfo");
            
            DbaseInfo dd = d.DbInfo;
            dd.DbaseName = "test2";
            
            if (ctx.Database.CeSetDatabaseInfo (d.Oid, dd))
               Console.WriteLine ("Database \"{0}\" renamed to \"{1}\"", test_db_name, dd.DbaseName);
            else
               Console.WriteLine ("Database \"{0}\" not renamed to \"{1}\"", test_db_name, dd.DbaseName);
         }            
      }
#endif

#if test1
      PrintName ("CeFindFirstDatabase");
      
      if ((find_handle = ctx.Database.CeFindFirstDatabase (0)) != DbFindHandle.Zero) {
         while ((oid = ctx.Database.CeFindNextDatabase (find_handle)) != Oid.Zero) {
            Console.WriteLine ("----------");
            Console.WriteLine ("  Oid: \"{0}\"", oid.Value);
         }

         ctx.Filesystem.CeCloseHandle (find_handle);
         Console.WriteLine ("Find handle closed");
      }

      PrintName ("CeCreateDatabase");
      
      test_db_name = "test";
      
      if ((oid = ctx.Database.CeCreateDatabase (test_db_name, 0, null)) != Oid.Zero)
         Console.WriteLine ("Database \"{0}\" created", test_db_name);
      else
         Console.WriteLine ("Database \"{0}\" not created", test_db_name);

      PrintName ("CeDeleteDatabase");
      
      if (ctx.Database.CeDeleteDatabase (oid))
         Console.WriteLine ("Database \"{0}\" deleted", test_db_name);
      else
         Console.WriteLine ("Database \"{0}\" not deleted", test_db_name);
#endif

#if test2
      PrintName ("CeOpenDatabase");
      
      DbHandle test_handle = DbHandle.Zero;
      test_db_name = "Contacts Database";
      
      test_handle = ctx.Database.CeOpenDatabase (Oid.Zero, test_db_name, PropId.Zero, flags);
      
      if (test_handle != DbHandle.Zero) {
         Console.WriteLine ("Database \"{0}\" opened", test_db_name);

         PropVal[] props = null;
         
         PrintName ("CeSeekDatabase");
         
         uint index = 0;

         oid = ctx.Database.CeSeekDatabase (test_handle, DatabaseSeekType.Beginning, 1, out index);
         
         if (oid != Oid.Zero)
            Console.WriteLine ("Database seeked to oid {0} index {1}", oid.Value, index);
         else
            Console.WriteLine ("Database not seeked to oid {0} index {1}", oid.Value, index);
         
         PrintName ("CeReadRecordProps");
         
         while ((oid = ctx.Database.CeReadRecordProps (test_handle, flags, out props)) != Oid.Zero) {
            Console.WriteLine ("For oid {0} readed {1} props", oid.Value, props.Length);
            
            foreach (PropVal p in props) {
               string val = "";
               
               switch (p.PropId.ValType) {
               case PropValType.Int16: val = p.Int16.ToString (); break;
               case PropValType.Int32: val = p.Int32.ToString (); break;
               case PropValType.Double: val = p.Double.ToString (); break;
               case PropValType.Bool: val = p.Bool.ToString (); break;
               case PropValType.UInt16: val = p.UInt16.ToString (); break;
               case PropValType.UInt32: val = p.UInt32.ToString (); break;
               case PropValType.String: val = p.String.ToString (); break;
               case PropValType.Filetime: val = p.Filetime.DateTime.ToString (); break;
               case PropValType.Blob: 
                  for (int i = 0; i < p.Blob.Length; i++)
                     val += (val.Length > 0 ? " " : "") + p.Blob [i].ToString ("X2");
                  break;
               }

               Console.WriteLine ("  ---------");
               Console.WriteLine ("    Id: 0x{0} Type: {1}", p.PropId.Id.ToString ("X4"), p.PropId.ValType);
               Console.WriteLine ("    Val: \"{0}\" size: {1}", val, val.Length);
            }
         }
            
         if (ctx.Filesystem.CeCloseHandle (test_handle))
            Console.WriteLine ("Database \"{0}\" closed", test_db_name);
         else
            Console.WriteLine ("Failed to close database \"{0}\"", test_db_name);
      }
#endif

#if test3
      DbHandle db_handle = DbHandle.Zero;
      test_db_name = "test2";
      
      if ((db_handle = ctx.CeOpenDatabase (Oid.Zero, test_db_name, PropId.Zero, flags)) != DbHandle.Zero) {
         Console.WriteLine ("Database \"{0}\" opened", test_db_name);

         oid = Oid.Zero;
         
         PropVal[] props = new PropVal [11];
         
         props [0].PropId = new PropId (0x0101, PropValType.UInt32);
         props [0].UInt32 = 0x01020304;
         props [1].PropId = new PropId (0x0202, PropValType.String);
         props [1].String = "test";
         props [2].PropId = new PropId (0x0303, PropValType.Blob);
         props [2].Blob = new byte [4] {1, 2, 3, 4};
         props [3].PropId = new PropId (0x0404, PropValType.String);
         props [3].String = "test2";
         props [4].PropId = new PropId (0x0505, PropValType.Blob);
         props [4].Blob = new byte [4] {0x11, 0x12, 0x13, 0x14};
         props [5].PropId = new PropId (0x0606, PropValType.UInt16);
         props [5].UInt16 = 0x0102;
         props [6].PropId = new PropId (0x0707, PropValType.Int32);
         props [6].Int32 = 0x11223344;
         props [7].PropId = new PropId (0x0808, PropValType.Int16);
         props [7].Int16 = 0x1122;
         props [8].PropId = new PropId (0x0909, PropValType.String);
         props [8].String = "test3";
         props [9].PropId = new PropId (0x1010, PropValType.Double);
         props [9].Double = 123.123;
         props [10].PropId = new PropId (0x1111, PropValType.Bool);
         props [10].Bool = true;
         
         if (ctx.CeWriteRecordProps (db_handle, ref oid, props)) {
            Console.WriteLine ("Record created");
         
            if (ctx.CeDeleteRecord (db_handle, oid))
               Console.WriteLine ("Record deleted");
            else
               Console.WriteLine ("Record not deleted");
         } else
            Console.WriteLine ("Record not created");
         
         if (ctx.CeCloseHandle (db_handle))
            Console.WriteLine ("Database \"{0}\" closed", test_db_name);
         else
            Console.WriteLine ("Failed to close database \"{0}\"", test_db_name);
      } else
         Console.WriteLine ("Database \"{0}\" not opened", test_db_name);
#endif

      connection.DestroyContext (ctx);
      connection.Dispose ();
   }
}
