/*
 * $Id: TestFilesystem.cs 289 2007-06-12 09:27:37Z lloydd $
 */

#define test2
 
using System;
using System.Threading;
using WiMo.RAPI;

public class TestSystemClass
{
   public static void PrintName (string s)
   {
      Console.WriteLine ("---------------------------");
      Console.WriteLine ("--- {0}", s);
      Console.WriteLine ("---------------------------");
   }
   
   public static void Main (string[] args)
   {
      Connection connection = null;
      Context ctx = null;
      
      try {
         Console.WriteLine ("Connecting...");
         
         connection = new Connection ("127.0.0.1", 10000);
         ctx = connection.CreateContext ();
      }
      catch (Exception e) {
         Console.Error.WriteLine ("Connect failed: {0}", e.Message);
         return;
      }
      
      Console.WriteLine ("Connected");

      FileHandle file_handle = FileHandle.Zero;
      string my_doc = ctx.CeGetSpecialFolderPath (SpecialFolder.MyDocuments);
      string test_file = "";

      FindFileAttribute flags = FindFileAttribute.Attributes|
                                FindFileAttribute.Creation_Time|
                                FindFileAttribute.Last_Access_Time|
                                FindFileAttribute.Last_Write_Time|
                                FindFileAttribute.Size_High|
                                FindFileAttribute.Size_Low|
                                FindFileAttribute.Oid|
                                FindFileAttribute.Name|
                                FindFileAttribute.Attrib_Children|
                                FindFileAttribute.Attrib_No_Hidden;

#if test0      
      PrintName ("CeCreateDirectory");
      
      string test_dir = "/My Documents/test";
      
      if (ctx.CeCreateDirectory (test_dir))
         Console.WriteLine ("Directory \"{0}\" created", test_dir);
      else
         Console.WriteLine ("Directory \"{0}\" not created", test_dir);

      PrintName ("CeRemoveDirectory");
      
      if (ctx.CeRemoveDirectory (test_dir))
         Console.WriteLine ("Directory \"{0}\" removed", test_dir);
      else
         Console.WriteLine ("Directory \"{0}\" not removed", test_dir);
         
      PrintName ("CeGetSpecialFolderPath");
      
      string dir = "";

      foreach (SpecialFolder s in Enum.GetValues (typeof (SpecialFolder))) {
         dir = ctx.CeGetSpecialFolderPath (s);
         Console.WriteLine ("\"{0}\" is \"{1}\"",
            s.ToString ().Replace ('_', ' '),
            dir);
      }
#endif

#if test1
      PrintName ("CeDeleteFile");

      file_handle = FileHandle.Zero;
      test_file = my_doc + "\\test_file.txt";
      
      if (ctx.CeDeleteFile (test_file))
         Console.WriteLine ("File \"{0}\" deleted", test_file);
      else
         Console.WriteLine ("File \"{0}\" not deleted", test_file);

      PrintName ("CeCreateFile");
      
      file_handle = ctx.CeCreateFile (test_file,
                                       ShareMode.Generic_Write|ShareMode.Generic_Read,
                                       FileShareMode.Share_Write|FileShareMode.Share_Read,
                                       CreationDisposition.Open_Always,
                                       FileAttribute.Archive,
                                       FileHandle.Zero);

      if (file_handle != FileHandle.Zero) {
         Console.WriteLine ("File \"{0}\" created", test_file);
            
         PrintName ("CeWriteFile");
         
         byte[] buf = new byte [4] {(byte) 'a', (byte) 'h', (byte) 'o', (byte) 'j'};
         uint bytes_written = 0;
         
         bytes_written = ctx.CeWriteFile (file_handle, buf);
         Console.WriteLine ("To file \"{0}\" written {1} B", test_file, bytes_written);
         
         PrintName ("CeGetFileSize");
         
         uint file_size_high = 0;
         uint file_size_low = 0;
         
         if (ctx.CeGetFileSize (file_handle, out file_size_high, out file_size_low))
            Console.WriteLine ("Size of file \"{0}\" is {1} B", test_file,
               file_size_high,
               file_size_low);
         else
            Console.WriteLine ("Failed to get size of file \"{0}\"", test_file);
            
         PrintName ("CeCloseHandle");
         
         if (ctx.CeCloseHandle (file_handle))
            Console.WriteLine ("File \"{0}\" closed", test_file);
         else
            Console.WriteLine ("File \"{0}\" not closed", test_file);
      }
      
      PrintName ("CeCopyFile");

      string new_test_file = test_file + ".test";
      
      if (ctx.CeCopyFile (test_file, new_test_file, false))
         Console.WriteLine ("File \"{0}\" copyied to \"{1}\"", test_file, new_test_file);
      else
         Console.WriteLine ("File \"{0}\" not copyied to \"{1}\"", test_file, new_test_file);

      PrintName ("CeMoveFile");

      string new_test_file2 = new_test_file + ".test";
      
      if (ctx.CeMoveFile (test_file, new_test_file2))
         Console.WriteLine ("File \"{0}\" moved to \"{1}\"", test_file, new_test_file2);
      else
         Console.WriteLine ("File \"{0}\" not moved to \"{1}\"", test_file, new_test_file2);

      PrintName ("CeDeleteFile");
      
      if (ctx.CeDeleteFile (new_test_file))
         Console.WriteLine ("File \"{0}\" deleted", new_test_file);
      else
         Console.WriteLine ("File \"{0}\" not deleted", new_test_file);

      PrintName ("CeDeleteFile");
      
      if (ctx.CeDeleteFile (new_test_file2))
         Console.WriteLine ("File \"{0}\" deleted", new_test_file2);
      else
         Console.WriteLine ("File \"{0}\" not deleted", new_test_file2);
#endif

#if test2
      PrintName ("CeFindAllFiles");
      
      string find_dir = my_doc + "\\*.*";
      
      FindData[] data = ctx.CeFindAllFiles (find_dir, flags);
      
      Console.WriteLine ("In directory \"{0}\" found {1} files", find_dir, data.Length);
         
      foreach (FindData fd in data) {
         Console.WriteLine ("----------");
         Console.WriteLine ("  FileAttributes: \"{0}\"", fd.FileAttributes);
         Console.WriteLine ("  CreationTime: \"{0}\"", fd.CreationTime.DateTime);
         Console.WriteLine ("  LastAccessTime: \"{0}\"", fd.LastAccessTime.DateTime);
         Console.WriteLine ("  LastWriteTime: \"{0}\"", fd.LastWriteTime.DateTime);
         Console.WriteLine ("  FileSizeHigh: \"{0}\"", fd.FileSizeHigh);
         Console.WriteLine ("  FileSizeLow: \"{0}\"", fd.FileSizeLow);
         Console.WriteLine ("  Oid: \"{0}\"", fd.Oid.Value);
         Console.WriteLine ("  FileName: \"{0}\"", fd.FileName);
      }
      
      PrintName ("CeFindFirstFile");
      
      string find_files = my_doc + "\\*.pwi";
      FindData ffd;
      FileHandle handle = FileHandle.Zero;
      FileFindHandle first_handle = ctx.CeFindFirstFile (find_files, out ffd);
      
      if (first_handle != FileFindHandle.Zero) {
         Console.WriteLine ("----------");
         Console.WriteLine ("  FileAttributes: \"{0}\"", ffd.FileAttributes);
         Console.WriteLine ("  CreationTime: \"{0}\"", ffd.CreationTime.DateTime);
         Console.WriteLine ("  LastAccessTime: \"{0}\"", ffd.LastAccessTime.DateTime);
         Console.WriteLine ("  LastWriteTime: \"{0}\"", ffd.LastWriteTime.DateTime);
         Console.WriteLine ("  FileSizeHigh: \"{0}\"", ffd.FileSizeHigh);
         Console.WriteLine ("  FileSizeLow: \"{0}\"", ffd.FileSizeLow);
         Console.WriteLine ("  Oid: \"{0}\"", ffd.Oid.Value);
         Console.WriteLine ("  FileName: \"{0}\"", ffd.FileName);

         PrintName ("CeFindNextFile");
         
         while ((handle = ctx.CeFindNextFile (first_handle, out ffd)) != FileHandle.Zero) {
            Console.WriteLine ("----------");
            Console.WriteLine ("  FileAttributes: \"{0}\"", ffd.FileAttributes);
            Console.WriteLine ("  CreationTime: \"{0}\"", ffd.CreationTime.DateTime);
            Console.WriteLine ("  LastAccessTime: \"{0}\"", ffd.LastAccessTime.DateTime);
            Console.WriteLine ("  LastWriteTime: \"{0}\"", ffd.LastWriteTime.DateTime);
            Console.WriteLine ("  FileSizeHigh: \"{0}\"", ffd.FileSizeHigh);
            Console.WriteLine ("  FileSizeLow: \"{0}\"", ffd.FileSizeLow);
            Console.WriteLine ("  Oid: \"{0}\"", ffd.Oid.Value);
            Console.WriteLine ("  FileName: \"{0}\"", ffd.FileName);
         }
      }
         
      PrintName ("CeFindClose");
      
      if (ctx.CeFindClose (first_handle))
         Console.WriteLine ("Find handle {0} closed", first_handle);
      else
         Console.WriteLine ("Find handle {0} not closed", first_handle);
#endif

#if test3
      PrintName ("CeGetFileAttributes");
      
      test_file = my_doc + "\\Test.txt";
      FileAttribute fa;
      
      if (ctx.Filesystem.CeGetFileAttributes (test_file, out fa))
         Console.WriteLine ("File {0} attribute {1}", test_file, fa);
      else
         Console.WriteLine ("Failed to get file {0} attribute", test_file);

      PrintName ("CeSetFileAttributes");
      
      fa = FileAttribute.Archive;
      
      if (ctx.Filesystem.CeSetFileAttributes (test_file, fa))
         Console.WriteLine ("File {0} attribute set to {1}", test_file, fa);
      else
         Console.WriteLine ("Failed to set file {0} attribute set to {1}", test_file, fa);
#endif

#if test4
      PrintName ("CeSHCreateShortcut");
      
      string shortcut = my_doc + "\\test.lnk";
      test_file = my_doc + "\\Test.txt";
      
      if (ctx.Filesystem.CeSHCreateShortcut (shortcut, test_file))
         Console.WriteLine ("Created shortcut {0} to {1}", shortcut, test_file);
      else
         Console.WriteLine ("Failed to created shortcut {0} to {1}", shortcut, test_file);
#endif

#if test5
      PrintName ("CeCreateFile");
      
      test_file = my_doc + "\\Test2.txt";
      
      file_handle = ctx.Filesystem.CeCreateFile (test_file,
                                       ShareMode.Generic_Write|ShareMode.Generic_Read,
                                       FileShareMode.Share_Write|FileShareMode.Share_Read,
                                       CreationDisposition.Open_Existing,
                                       FileAttribute.Archive,
                                       FileHandle.Zero);
      if (file_handle != FileHandle.Zero) {
         Console.WriteLine ("File \"{0}\" opened", test_file);
   
         PrintName ("CeGetFileSize");
   
         uint file_size_high = 0;
         uint file_size_low = 0;
         
         if (ctx.Filesystem.CeGetFileSize (file_handle, out file_size_high, out file_size_low))
            Console.WriteLine ("Size of file \"{0}\" is {1}-{2} B", test_file,
                  file_size_high, file_size_low);
         else
            Console.WriteLine ("Failed to get size of file \"{0}\"", test_file);
         
         PrintName ("CeReadFile");
         
         byte[] buf = null;
         uint bytes_readed = ctx.Filesystem.CeReadFile (file_handle, out buf, 1024);
         Console.WriteLine ("From file \"{0}\" readed {1} B", test_file, bytes_readed);
         
         Console.WriteLine ("Data: ");
         for (int i = 0; i < bytes_readed; i++)
            Console.Write ((char) buf [i]);
         Console.WriteLine ();
         
         PrintName ("CeCloseHandle");
         
         if (ctx.Filesystem.CeCloseHandle (file_handle))
            Console.WriteLine ("File \"{0}\" closed", test_file);
         else
            Console.WriteLine ("File \"{0}\" not closed", test_file);
      }
#endif      
         
      connection.DestroyContext (ctx);
      connection.Dispose ();
   }
}
