/*
 * $Id: TestSystem.cs 289 2007-06-12 09:27:37Z lloydd $
 */
 
#define test1
 
using System;
using System.Threading;
using WiMo.RAPI;

using System.IO;
using System.Net;
using System.Net.Sockets;

public class TestSystemClass
{
   public static void PrintName (string s)
   {
      Console.WriteLine ("---------------------------");
      Console.WriteLine ("--- {0}", s);
      Console.WriteLine ("---------------------------");
   }

   public static void Main (string[] args)
   {
      Connection connection = null;
      Context ctx = null;
      
      try {
         Console.WriteLine ("Connecting...");
         
         connection = new Connection ("127.0.0.1", 10000);
         ctx = connection.CreateContext ();
      }
      catch (Exception e) {
         Console.Error.WriteLine ("Connect failed: {0}", e.Message);
         return;
      }
      
      Console.WriteLine ("Connected");
      
      StoreInformation si;
      SystemInfo sy;
      SystemPowerStatusEx ps;
      VersionInfo vi;

      PrintName ("CeCheckPassword");
      
      if (ctx.CeCheckPassword ("password"))
         Console.WriteLine ("Password is ok");
      else
         Console.WriteLine ("Password is bad");
         
      PrintName ("CeGetStoreInformation");
      
      if (ctx.CeGetStoreInformation (out si))
         Console.WriteLine ("free: {0} store: {1}", si.FreeSize, si.StoreSize);
      else
         Console.WriteLine ("CeGetStoreInformation failed");

      PrintName ("CeGetSystemInfo");
         
      sy = ctx.CeGetSystemInfo ();
      Console.WriteLine ("ProcessorArch: {0}", sy.ProcessorArch);
      Console.WriteLine ("Reserved: {0}", sy.Reserved);
      Console.WriteLine ("PageSize: {0}", sy.PageSize);
      Console.WriteLine ("MinimumApplicationAddress: {0}", sy.MinimumApplicationAddress);
      Console.WriteLine ("MaximumApplicationAddress: {0}", sy.MaximumApplicationAddress);
      Console.WriteLine ("ActiveProcessorMask: {0}", sy.ActiveProcessorMask);
      Console.WriteLine ("NumberOfProcessors: {0}", sy.NumberOfProcessors);
      Console.WriteLine ("ProcessorType: {0}", sy.ProcessorType);
      Console.WriteLine ("AllocationGranularity: {0}", sy.AllocationGranularity);
      Console.WriteLine ("ProcessorLevel: {0}", sy.ProcessorLevel);
      Console.WriteLine ("ProcessorRevision: {0}", sy.ProcessorRevision);

      PrintName ("CeGetSystemPowerStatusEx");
         
      if (ctx.CeGetSystemPowerStatusEx (out ps, true)) {
         Console.WriteLine ("AcLineStatus: {0}", ps.AcLineStatus);
         Console.WriteLine ("BatteryFlag: {0}", ps.BatteryFlag);
         Console.WriteLine ("BatteryLifePercent: {0}", ps.BatteryLifePercent);
         Console.WriteLine ("Reserved1: {0}", ps.Reserved1);
         Console.WriteLine ("BatteryLifeTime: {0}", ps.BatteryLifeTime);
         Console.WriteLine ("BatteryFullLifeTime: {0}", ps.BatteryFullLifeTime);
         Console.WriteLine ("Reserved2: {0}", ps.Reserved2);
         Console.WriteLine ("BackupBatteryFlag: {0}", ps.BackupBatteryFlag);
         Console.WriteLine ("BackupBatteryLifePercent: {0}", ps.BackupBatteryLifePercent);
         Console.WriteLine ("Reserved3: {0}", ps.Reserved3);
         Console.WriteLine ("BackupBatteryLifeTime: {0}", ps.BackupBatteryLifeTime);
         Console.WriteLine ("BackupBatteryFullLifeTime: {0}", ps.BackupBatteryFullLifeTime);
      } else
         Console.WriteLine ("CeGetSystemPowerStatusEx failed");

      PrintName ("CeGetVersionEx");
         
      if (ctx.CeGetVersionEx (out vi)) {
         Console.WriteLine ("MajorVersion: {0}", vi.MajorVersion);
         Console.WriteLine ("MinorVersion: {0}", vi.MinorVersion);
         Console.WriteLine ("BuildNumber: {0}", vi.BuildNumber);
         Console.WriteLine ("Platform: {0}", vi.Platform);
      } else
         Console.WriteLine ("CeGetSystemInfo failed");

      PrintName ("CeGetSystemMemoryDivision");
      
      uint storage_pages = 0;
      uint ram_pages = 0;
      uint page_size = 0; 
      
      if (ctx.CeGetSystemMemoryDivision (out storage_pages,
                                            out ram_pages,
                                            out page_size)) {
         Console.WriteLine ("storage_pages: {0}", storage_pages);
         Console.WriteLine ("ram_pages: {0}", ram_pages);
         Console.WriteLine ("page_size: {0}", page_size);
      } else
         Console.WriteLine ("CeGetSystemMemoryDivision failed");

      PrintName ("CeSetSystemMemoryDivision");
      
      if (ctx.CeSetSystemMemoryDivision (storage_pages + 10))
         Console.WriteLine ("System memory division set to {0}", storage_pages + 10);
      else
         Console.WriteLine ("CeSetSystemMemoryDivision failed");

      PrintName ("CeGetSystemMemoryDivision");
      
      storage_pages = 0;
      ram_pages = 0;
      page_size = 0; 
      
      if (ctx.CeGetSystemMemoryDivision (out storage_pages,
                                            out ram_pages,
                                            out page_size)) {
         Console.WriteLine ("storage_pages: {0}", storage_pages);
         Console.WriteLine ("ram_pages: {0}", ram_pages);
         Console.WriteLine ("page_size: {0}", page_size);
      } else
         Console.WriteLine ("CeGetSystemMemoryDivision failed");

      PrintName ("CeSetSystemMemoryDivision");
      
      if (ctx.CeSetSystemMemoryDivision (storage_pages - 10))
         Console.WriteLine ("System memory division set to {0}", storage_pages - 10);
      else
         Console.WriteLine ("CeSetSystemMemoryDivision failed");

#if test9
      PrintName ("CeSyncTimeToPc");

      ctx.CeSyncTimeToPc ();
#endif

      PrintName ("CeRegCopyFile");

      string reg_file = @"\My Documents\test.reg";
      
      if (ctx.CeRegCopyFile (reg_file))
         Console.WriteLine ("Registers saved to \"{0}\"", reg_file);
      else
         Console.WriteLine ("Failed to save registers to \"{0}\"", reg_file);

#if test11
      PrintName ("CeKillAllApps");

      if (ctx.CeKillAllApps ())
         Console.WriteLine ("All applications killed");
      else
         Console.WriteLine ("Failed to killa all applications");
#endif

      PrintName ("CeCreateProcess");

      string app = @"\My Documents\Programs\cabinstl.exe";
      string cmd = "";
      ProcessInformation process_info;
      
      if (ctx.CeCreateProcess (app, cmd, out process_info)) {
         Console.WriteLine ("Application executed {0} {1}", app, cmd);
         Console.WriteLine ("---------");
         Console.WriteLine ("  Process: 0x{0}", process_info.Process.ToString ("X8"));
         Console.WriteLine ("  Thread: 0x{0}", process_info.Thread.ToString ("X8"));
         Console.WriteLine ("  ProcessId: 0x{0}", process_info.ProcessId.ToString ("X8"));
         Console.WriteLine ("  ThreadId: 0x{0}", process_info.ThreadId.ToString ("X8"));
      } else
         Console.WriteLine ("CeCreateProcess failed");

      connection.DestroyContext (ctx);
      connection.Dispose ();
   }
}
