/*
 * $Id: TestFilesystem.cs 120 2007-04-04 14:15:38Z lloydd $
 */

using System;
using System.IO;
using System.Threading;
using RAPI;

public class TestSystemClass
{
   public static void PrintName (string s)
   {
      Console.WriteLine ("---------------------------");
      Console.WriteLine ("--- {0}", s);
      Console.WriteLine ("---------------------------");
   }
   
   public static void Main (string[] args)
   {
      Connection connection = null;
      Context ctx = null;
      
      try {
         Console.WriteLine ("Connecting...");
         
         connection = new Connection ("10.100.1.2");
         ctx = connection.CreateContext ();
      }
      catch (Exception e) {
         Console.Error.WriteLine ("Connect failed: {0}", e.Message);
         return;
      }
      
      Console.WriteLine ("Connected");

      FileHandle file_handle = FileHandle.Zero;
      string my_doc = ctx.Filesystem.CeGetSpecialFolderPath (SpecialFolder.MyDocuments);
      string test_file = "";

      FindFileAttribute flags = FindFileAttribute.Attributes|
                                FindFileAttribute.Creation_Time|
                                FindFileAttribute.Last_Access_Time|
                                FindFileAttribute.Last_Write_Time|
                                FindFileAttribute.Size_High|
                                FindFileAttribute.Size_Low|
                                FindFileAttribute.Oid|
                                FindFileAttribute.Name|
                                FindFileAttribute.Attrib_Children|
                                FindFileAttribute.Attrib_No_Hidden;

      PrintName ("CeCreateFile");
      
      test_file = "f.zip";
      
      file_handle = ctx.Filesystem.CeCreateFile (test_file,
                                       ShareMode.Generic_Write|ShareMode.Generic_Read,
                                       FileShareMode.Share_Write|FileShareMode.Share_Read,
                                       CreationDisposition.Open_Existing,
                                       FileAttribute.Archive,
                                       FileHandle.Zero);
      if (file_handle != FileHandle.Zero) {
         Console.WriteLine ("File \"{0}\" opened", test_file);
   
         PrintName ("CeGetFileSize");
   
         uint file_size_high = 0;
         uint file_size_low = 0;
         
         if (ctx.Filesystem.CeGetFileSize (file_handle, out file_size_high, out file_size_low))
            Console.WriteLine ("Size of file \"{0}\" is {1}-{2} B", test_file,
                  file_size_high, file_size_low);
         else
            Console.WriteLine ("Failed to get size of file \"{0}\"", test_file);
         
         PrintName ("CeReadFile");
         
         byte[] buf = null;
         uint bytes_readed = ctx.Filesystem.CeReadFile (file_handle, out buf, 128*1024);
         Console.WriteLine ("From file \"{0}\" readed {1} B", test_file, bytes_readed);

         StreamWriter sw = new StreamWriter ("f.zip");
         BinaryWriter bw = new BinaryWriter (sw.BaseStream);
         bw.Write (buf);
         bw.Close ();
         sw.Close ();
         
         PrintName ("CeCloseHandle");
         
         if (ctx.Filesystem.CeCloseHandle (file_handle))
            Console.WriteLine ("File \"{0}\" closed", test_file);
         else
            Console.WriteLine ("File \"{0}\" not closed", test_file);
      }
         
      connection.DestroyContext (ctx);
      connection.Dispose ();
   }
}
